@@ -24,10 +24,15 @@ extern SEXP Rf_installTrChar(SEXP x);
 extern void R_removeVarFromFrame(SEXP name, SEXP env);
 
 extern SEXP R_NewEnv(SEXP enclos, int hash, int size);
-#if R_version_less_than(4,1,0) || (!defined(R_THIS_PATH_DEVEL) && R_version_at_least(4,5,0))
-#define IS_ASCII my_IS_ASCII
-#endif
+#if R_version_at_least(4,1,0) && (defined(R_THIS_PATH_DEVEL) || R_version_less_than(4,5,0))
 extern int IS_ASCII(SEXP x);
+#elif R_version_at_least(4,5,0)
+#define IS_ASCII Rf_charIsASCII
+#else
+#define R_THIS_PATH_NEED_IS_ASCII
+#define IS_ASCII my_IS_ASCII
+extern int IS_ASCII(SEXP x);
+#endif
 
 extern Rboolean R_existsVarInFrame(SEXP rho, SEXP symbol);
 
@@ -187,9 +192,14 @@ extern void (ENSURE_NAMEDMAX)(SEXP x);
 
 #if R_version_less_than(4,1,0) || (!defined(R_THIS_PATH_DEVEL) && R_version_at_least(4,5,0))
 #define IS_UTF8(x) (Rf_getCharCE((x)) == CE_UTF8)
-#define IS_LATIN1(x) (Rf_getCharCE((x)) == CE_LATIN1)
 #else
 extern int IS_UTF8(SEXP x);
+#endif
+
+
+#if R_version_less_than(4,1,0) || R_version_at_least(4,4,0) || !defined(R_THIS_PATH_DEVEL)
+#define IS_LATIN1(x) (Rf_getCharCE((x)) == CE_LATIN1)
+#else
 extern int IS_LATIN1(SEXP x);
 #endif
 
@@ -206,4 +216,32 @@ extern Rboolean Rf_isValidStringF(SEXP);
 #endif
 
 
+#if !defined(R_THIS_PATH_DEVEL) && R_version_at_least(4,5,0)
+#define OBJECT Rf_isObject
+#define IS_S4_OBJECT Rf_isS4
+#endif
+
+
+#if !defined(R_THIS_PATH_DEVEL) && R_version_at_least(4,5,0)
+#define BODY R_ClosureBody
+#define FORMALS R_ClosureFormals
+#define CLOENV R_ClosureEnv
+#endif
+
+
+#if !defined(R_THIS_PATH_DEVEL) && R_version_at_least(4,5,0)
+#define ENCLOS R_ParentEnv
+#endif
+
+
+#if R_version_less_than(4,5,0)
+#define NO_ATTRIB(x) (ATTRIB((x)) == R_NilValue)
+#endif
+
+
+#if R_version_less_than(4,5,0)
+#define CLEAR_ATTRIB(x) (SET_ATTRIB((x), R_NilValue))
+#endif
+
+
 #endif /* R_THISPATH_BACKPORTS_H */
