@@ -5,6 +5,15 @@
 #define debug 0
 
 
+#define NEED_TRAILING_SLASH_MASK   (1<<1)
+#define USE_MASK                   (1<<2)
+#define NEED_TRAILING_SLASH(x)     ((x) & NEED_TRAILING_SLASH_MASK)
+#define SET_NEED_TRAILING_SLASH(x) ((x) |= NEED_TRAILING_SLASH_MASK)
+#define IS_USE(x)                  ((x) & USE_MASK)
+#define SET_USE(x)                 ((x) |= USE_MASK)
+typedef unsigned char info_t;
+
+
 
 
 
@@ -14,23 +23,36 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
     int i, j;
 
 
-    int drive_indx,                /*  largest index of the string vector which contains a drivespec */
-        abs_path_indx,             /*  largest index of the string vector which contains an absolute pathspec */
-        non_empty_path_spec_indx;  /* smallest index of the string vector which contains a non-empty pathspec, but always >= abs_path_indx */
+        /*  largest index that contains a drivespec */
+    int drive_indx,
+        /*  largest index that contains an absolute pathspec */
+        abs_path_indx,
+        /* smallest index that contains a non-empty pathspec and is >= abs_path_indx */
+        non_empty_path_spec_indx;
 
 
     /* width of the path and of the drivespec */
     int pwidth, drivewidth;
 
 
-    const char *ptr;  /* points to the entire string from Rf_translateCharUTF8() */
-    int *need_trailing_slash;  /* which strings need a slash afterward */
-    need_trailing_slash = (int *) R_alloc(x_length, sizeof(int));
+    const char *str;
+    info_t *info;  /* which strings need a slash afterward and which are used */
+    info = (info_t *) R_alloc(x_length, sizeof(info_t));
 
 
     int len, nchar;
 
 
+    Rboolean is_latin1locale = my_latin1locale,
+             is_utf8locale   = my_utf8locale  ;
+
+
+    Rboolean allKnown, anyKnown, use_UTF8, use_Bytes;
+
+
+    SEXP cs;
+
+
     for (j = 0; j < commonLength; j++) {
 
 
@@ -39,30 +61,46 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
         non_empty_path_spec_indx = -1;
 
 
-        pwidth = 0;
         drivewidth = 0;
 
 
-        ptr = "";  /* for -Wall */
-        memset(need_trailing_slash, 0, x_length * sizeof(int));
+        memset(info, 0, x_length * sizeof(info_t));
 
 
-        /* we're going to work backwards here
-         * since we want the LAST absolute path
+        anyKnown = FALSE; allKnown = TRUE; use_UTF8 = FALSE; use_Bytes = FALSE;
+
+
+        /* find the desired encoding
+         * and the required indexes
+         *
+         * we're going to work backwards here
          */
         for (i = x_length - 1; i >= 0; i--) {
             len = LENGTH(VECTOR_ELT(x, i));
-            ptr = Rf_translateCharUTF8(STRING_ELT(VECTOR_ELT(x, i), j % len));
-            nchar = (int) strlen(ptr);
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            str = R_CHAR(cs);
+            nchar = (int) strlen(str);
             if (!nchar)
                 continue;
 
 
-            /* look for a drivespec in ptr */
-            drivewidth = drive_width_windows(ptr, nchar);
+            /* look for a drivespec */
+            drivewidth = drive_width_windows(str);
 
 
-            /* if we have no already found an absolute pathspec,
+            if (abs_path_indx == -1 || drivewidth) {
+                /* mark this string as used
+                 * and make a note of its encoding
+                 */
+                SET_USE(info[i]);
+                if (!use_Bytes) {
+                    if (IS_BYTES(cs)) use_Bytes = TRUE;
+                    else if (!use_UTF8 && IS_UTF8(cs)) use_UTF8 = TRUE;
+                }
+            }
+
+
+            /* if we have not already found an absolute pathspec,
              * then look for one
              */
             if (abs_path_indx == -1) {
@@ -75,56 +113,21 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
                     /* if the start of the pathspec is slash or backslash
                      * then record this index as the last absolute path
                      */
-                    if (ptr[drivewidth] == '/' || ptr[drivewidth] == '\\')
+                    if (str[drivewidth] == '/' || str[drivewidth] == '\\')
                         abs_path_indx = i;
 
 
                     /* record this index as the first non-empty pathspec */
                     non_empty_path_spec_indx = i;
-
-
-                    /* if there are characters already in the buffer
-                     * and this pathspec does not end with slash nor backslash
-                     * then we will need to add one manually
-                     *
-                     * record this index as needing a trailing slash
-                     */
-                    if (pwidth &&
-                        ptr[nchar - 1] != '/' &&
-                        ptr[nchar - 1] != '\\')
-                    {
-                        need_trailing_slash[i] = 1;
-                        pwidth++;
-                    }
-
-
-                    /* add the number of chars in the pathspec
-                     * to the width of our output string
-                     */
-                    pwidth += nchar - drivewidth;
-                }
-
-
-                /* empty pathspec, non-empty drivespec,
-                 * if there are no characters in the buffer, do not add a trailing slash
-                 * if this is a drive, do not add a trailing slash
-                 * otherwise, add a trailing slash
-                 */
-                else if (pwidth && drivewidth != 2) {
-                    need_trailing_slash[i] = 1;
-                    pwidth++;
                 }
             }
 
 
             /* non-empty drivespec
              * record this index as the last index containing a drive
-             * add the number of chars in the drive to the buffer
-             * break immediately, will not be adding more to the path
              */
             if (drivewidth) {
                 drive_indx = i;
-                pwidth += drivewidth;
                 break;
             }
         }
@@ -134,7 +137,7 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
          * and our drivespec is a drive
          *
          * then we will want to go back slightly further
-         * incase there's more to add to the path
+         * in case there is more to add to the path
          *
          * this is because drives are not always absolute
          *
@@ -154,59 +157,80 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
         if (abs_path_indx == -1 && drivewidth == 2) {
 
 
-            /* use a bunch of maybe variables
+            /* use a bunch of 'maybe' variables
              * these may be used, depending on how the drives look
              */
 
 
-            int maybe_abs_path_indx = -1,
-                maybe_non_empty_path_spec_indx = -1;
+            int maybe_abs_path_indx = abs_path_indx,
+                maybe_non_empty_path_spec_indx = non_empty_path_spec_indx;
 
 
-            int maybe_pwidth = 0, maybe_drivewidth = 0;
-            const char *maybe_ptr = "";
+            int maybe_drivewidth = 0;
+            const char *maybe_str;
             int maybe_len, maybe_nchar;
 
 
-            const char drive_letter = toupper(ptr[0]);
+            const char drive_letter = toupper(str[0]);
+
+
+            Rboolean maybe_use_UTF8 = use_UTF8,
+                     maybe_use_Bytes = use_Bytes;
+
+
+            SEXP maybe_cs;
 
 
             for (i--; i >= 0; i--) {
                 maybe_len = LENGTH(VECTOR_ELT(x, i));
-                maybe_ptr = Rf_translateCharUTF8(STRING_ELT(VECTOR_ELT(x, i), j % maybe_len));
-                maybe_nchar = (int) strlen(maybe_ptr);
+                maybe_cs = STRING_ELT(VECTOR_ELT(x, i), j % maybe_len);
+                maybe_str = R_CHAR(maybe_cs);
+                maybe_nchar = (int) strlen(maybe_str);
                 if (!maybe_nchar)
                     continue;
 
 
-                /* look for a drivespec in ptr */
-                maybe_drivewidth = drive_width_windows(maybe_ptr, maybe_nchar);
+                /* look for a drivespec */
+                maybe_drivewidth = drive_width_windows(maybe_str);
 
 
                 if (maybe_drivewidth) {
+                    /* if the string has a drivespec, ensure it matches the
+                     * previous drivespec, break otherwise
+                     */
                     if (maybe_drivewidth != 2 ||
-                        toupper(maybe_ptr[0]) != drive_letter)
+                        toupper(maybe_str[0]) != drive_letter)
                     {
                         break;
                     }
                 }
 
 
-                /* if we have no already found an absolute pathspec,
+                /* if we have not already found an absolute pathspec,
                  * then look for one
                  */
                 if (maybe_abs_path_indx == -1) {
 
 
-                    /* a non-empty pathspec */
+                    /* non-empty pathspec */
                     if (maybe_drivewidth < maybe_nchar) {
 
 
+                        /* mark this string as used
+                         * and make a note of its encoding
+                         */
+                        SET_USE(info[i]);
+                        if (!maybe_use_Bytes) {
+                            if (IS_BYTES(maybe_cs)) maybe_use_Bytes = TRUE;
+                            else if (!maybe_use_UTF8 && IS_UTF8(maybe_cs)) maybe_use_UTF8 = TRUE;
+                        }
+
+
                         /* if the start of the pathspec is slash or backslash
                          * then record this index as the last absolute path
                          */
-                        if (maybe_ptr[maybe_drivewidth] == '/' ||
-                            maybe_ptr[maybe_drivewidth] == '\\')
+                        if (maybe_str[maybe_drivewidth] == '/' ||
+                            maybe_str[maybe_drivewidth] == '\\')
                         {
                             maybe_abs_path_indx = i;
                         }
@@ -214,63 +238,21 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
 
                         /* record this index as the first non-empty pathspec */
                         maybe_non_empty_path_spec_indx = i;
-
-
-                        /* if there are characters already in the buffer (excluding the drive)
-                         * or there are characters already in the maybe buffer
-                         * and this pathspec does not end with slash nor back
-                         * then we will need to add one manually
-                         *
-                         * record this index as needing a trailing slash
-                         */
-                        if ((pwidth > 2 || maybe_pwidth) &&
-                            maybe_ptr[maybe_nchar - 1] != '/' &&
-                            maybe_ptr[maybe_nchar - 1] != '\\')
-                        {
-                            need_trailing_slash[i] = 1;
-                            maybe_pwidth++;
-                        }
-
-
-                        /* add the number of chars in the pathspec
-                         * to the width of our output string
-                         */
-                        maybe_pwidth += maybe_nchar - maybe_drivewidth;
                     }
-                    /* this statement will always be false (since drivewidth = 2) */
-//
-//
-//                     /* empty pathspec, non-empty drivespec,
-//                      * if there are no characters in the buffer, do not add a trailing slash
-//                      * if this is a drive, do not add a trailing slash
-//                      * otherwise, add a trailing slash
-//                      */
-//                     else if (pwidth && drivewidth != 2) {
-//                         // Rprintf("needs a slash\n");
-//                         need_trailing_slash[i] = 1;
-//                         pwidth++;
-//                     }
                 }
 
 
                 /* non-empty drivespec that matches our previous drivespec
-                 * add the absolute path index and the non empty pathspec index
-                 * back into our original variables
-                 * also add the width to the original pwidth
                  *
-                 * if we have not found an absolute path, continue looking back
-                 * otherwise, we are done
+                 * copy the 'maybe' variables back into the original variables
+                 *
+                 * if we have not found an absolute path, continue looking
                  */
                 if (maybe_drivewidth) {
-                    if (maybe_non_empty_path_spec_indx != -1)
-                        non_empty_path_spec_indx = maybe_non_empty_path_spec_indx;
+                    non_empty_path_spec_indx = maybe_non_empty_path_spec_indx;
 
 
-                    pwidth += maybe_pwidth; maybe_pwidth = 0;
-
-
-                    /* unlike above, we are NOT adding the drive to the buffer */
-                    // pwidth += drivewidth;
+                    use_UTF8 = maybe_use_UTF8; use_Bytes = maybe_use_Bytes;
 
 
                     /* if we found an absolute path, then we are done */
@@ -283,6 +265,102 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
         }
 
 
+        /* find the minimum of two indexes, ignoring -1 */
+#define MIN_INDX(x, y)                                         \
+        (((x) != -1) ? (((y) != -1) ? (((x) < (y)) ? (x) : (y)) : (x)) : (y))
+
+
+        int path_min = MIN_INDX(abs_path_indx, non_empty_path_spec_indx);
+        int i_min = MIN_INDX(path_min, drive_indx);
+
+
+        /* if all the paths are empty */
+        if (i_min == -1)
+            continue;
+
+
+        pwidth = 0;
+
+
+        /* now that we have the desired encoding and the required indexes,
+         * find the required width
+         */
+        for (i = x_length - 1; i >= i_min; i--) {
+
+
+            /* if the string is not used, then no need to translate */
+            if (!IS_USE(info[i]))
+                continue;
+
+
+            len = LENGTH(VECTOR_ELT(x, i));
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            if (use_Bytes)
+                str = R_CHAR(cs);
+            else if (use_UTF8)
+                str = Rf_translateCharUTF8(cs);
+            else
+                str = Rf_translateChar(cs);
+            nchar = (int) strlen(str);
+            if (!nchar)
+                continue;
+
+
+            /* look for a drivespec */
+            drivewidth = drive_width_windows(str);
+
+
+            /* not already found an absolute pathspec */
+            if (i >= abs_path_indx) {
+
+
+                /* non-empty pathspec */
+                if (drivewidth < nchar) {
+
+
+                    /* if there are pathspec characters already in the buffer
+                     * and this pathspec does not end with slash nor backslash
+                     * then we will need to add one manually
+                     *
+                     * record this index as needing a trailing slash
+                     */
+                    if (pwidth > ((i >= drive_indx) ? 0 : 2) &&
+                        str[nchar - 1] != '/' &&
+                        str[nchar - 1] != '\\')
+                    {
+                        SET_NEED_TRAILING_SLASH(info[i]);
+                        pwidth++;
+                    }
+
+
+                    /* add the number of chars in the pathspec
+                     * to the width of the output string
+                     */
+                    pwidth += nchar - drivewidth;
+                }
+
+
+                /* empty pathspec, non-empty drivespec,
+                 * if there are no characters in the buffer, do not add a trailing slash
+                 * if this is a drive, do not add a trailing slash
+                 * otherwise, add a trailing slash
+                 */
+                else if (pwidth && drivewidth != 2) {
+                    SET_NEED_TRAILING_SLASH(info[i]);
+                    pwidth++;
+                }
+            }
+
+
+            /* non-empty drivespec
+             * add the number of chars in the drive to the buffer
+             */
+            if (i == drive_indx) {
+                pwidth += drivewidth;
+            }
+        }
+
+
         /* if all the strings were empty, return an empty string */
         if (pwidth <= 0) {
             // SET_STRING_ELT(value, j, R_BlankString);
@@ -310,32 +388,48 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
         const char *cbuf = buf;
 
 
-        int start_from_here = abs_path_indx;
-        if (start_from_here == -1)
-            start_from_here = non_empty_path_spec_indx;
+        int start_from_here = path_min;
 
 
         /* if we found a drive */
         if (drive_indx != -1) {
 
 
-            /* and the drive is at the same index as the start pos
+            i = drive_indx;
+            len = LENGTH(VECTOR_ELT(x, i));
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            if (use_Bytes)
+                str = R_CHAR(cs);
+            else if (use_UTF8)
+                str = Rf_translateCharUTF8(cs);
+            else {
+                str = Rf_translateChar(cs);
+                // allKnown = allKnown && (IS_ASCII(cs) || ENC_KNOWN(cs));
+                // anyKnown = anyKnown || ENC_KNOWN(cs);
+                allKnown = allKnown && (IS_ASCII(cs) || IS_LATIN1(cs));
+                anyKnown = anyKnown || IS_LATIN1(cs);
+            }
+            nchar = (int) strlen(str);
+            if (!nchar);
+
+
+            /* if drive is at the same index as the start pos
              * copy to the buffer without the need for splitting */
-            if (start_from_here == drive_indx) {
-                strcpy(buf, ptr);
+            else if (start_from_here == i) {
+                strcpy(buf, str);
                 buf += nchar;
 #if debug
-                Rprintf("copied to buffer: %s", ptr); Rprintf("\n");
+                Rprintf("copied to buffer: %s", str); Rprintf("\n");
                 Rprintf("shifted buffer by %d bytes\n", nchar);
-                check_width += strlen(ptr);
+                check_width += (int) strlen(str);
 #endif
-                if (need_trailing_slash[drive_indx]) {
+                if (NEED_TRAILING_SLASH(info[i])) {
                     strcpy(buf, "/");
                     buf++;
 #if debug
                     Rprintf("copied to buffer: %s", "/"); Rprintf("\n");
-                    Rprintf("shifted buffer by %d bytes\n", strlen("/"));
-                    check_width += strlen("/");
+                    Rprintf("shifted buffer by %d bytes\n", (int) strlen("/"));
+                    check_width += (int) strlen("/");
 #endif
                 }
                 start_from_here++;
@@ -346,25 +440,31 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
              * so copy only the drive
              */
             else {
-                strncpy(buf, ptr, drivewidth);
+
+
+                /* look for a drivespec */
+                drivewidth = drive_width_windows(str);
+
+
+                strncpy(buf, str, drivewidth);
                 buf += drivewidth;
                 *buf = '\0';  // copy a nul character
 #if debug
-                Rprintf("copied first %d bytes to buffer: %s", drivewidth, ptr); Rprintf("\n");
+                Rprintf("copied first %d bytes to buffer: %s", drivewidth, str); Rprintf("\n");
                 Rprintf("shifted buffer by %d bytes\n", drivewidth);
                 check_width += drivewidth;
 #endif
 
 
-                if (start_from_here > drive_indx &&
-                    need_trailing_slash[drive_indx])
+                if (start_from_here > i &&
+                    NEED_TRAILING_SLASH(info[i]))
                 {
                     strcpy(buf, "/");
                     buf++;
 #if debug
                     Rprintf("copied to buffer: %s", "/"); Rprintf("\n");
-                    Rprintf("shifted buffer by %d bytes\n", strlen("/"));
-                    check_width += strlen("/");
+                    Rprintf("shifted buffer by %d bytes\n", (int) strlen("/"));
+                    check_width += (int) strlen("/");
 #endif
                 }
             }
@@ -376,9 +476,26 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
 
             /* start immediately AFTER the absolute path */
             for (i = start_from_here; i < x_length; i++) {
+
+
+                if (!IS_USE(info[i]))
+                    continue;
+
+
                 len = LENGTH(VECTOR_ELT(x, i));
-                ptr = Rf_translateCharUTF8(STRING_ELT(VECTOR_ELT(x, i), j % len));
-                nchar = (int) strlen(ptr);
+                cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+                if (use_Bytes)
+                    str = R_CHAR(cs);
+                else if (use_UTF8)
+                    str = Rf_translateCharUTF8(cs);
+                else {
+                    str = Rf_translateChar(cs);
+                    // allKnown = allKnown && (IS_ASCII(cs) || ENC_KNOWN(cs));
+                    // anyKnown = anyKnown || ENC_KNOWN(cs);
+                    allKnown = allKnown && (IS_ASCII(cs) || IS_LATIN1(cs));
+                    anyKnown = anyKnown || IS_LATIN1(cs);
+                }
+                nchar = (int) strlen(str);
 
 
                 /* an empty string does not need to be copied to the buffer
@@ -388,12 +505,12 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
                     continue;
 
 
-                /* if we're before or at the drive indx,
+                /* if we are before or at the drive indx,
                  * be sure to chop off the drive before pasting
                  */
                 if (i <= drive_indx) {
-                    drivewidth = drive_width_windows(ptr, nchar);
-                    ptr += drivewidth;
+                    drivewidth = drive_width_windows(str);
+                    str += drivewidth;
                     nchar -= drivewidth;
 
 
@@ -402,20 +519,20 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
                 }
 
 
-                strcpy(buf, ptr);
+                strcpy(buf, str);
                 buf += nchar;
 #if debug
-                Rprintf("copied to buffer: %s", ptr); Rprintf("\n");
+                Rprintf("copied to buffer: %s", str); Rprintf("\n");
                 Rprintf("shifted buffer by %d bytes\n", nchar);
-                check_width += strlen(ptr);
+                check_width += (int) strlen(str);
 #endif
-                if (need_trailing_slash[i]) {
+                if (NEED_TRAILING_SLASH(info[i])) {
                     strcpy(buf, "/");
                     buf++;
 #if debug
                     Rprintf("copied to buffer: %s", "/"); Rprintf("\n");
-                    Rprintf("shifted buffer by %d bytes\n", strlen("/"));
-                    check_width += strlen("/");
+                    Rprintf("shifted buffer by %d bytes\n", (int) strlen("/"));
+                    check_width += (int) strlen("/");
 #endif
                 }
             }
@@ -424,13 +541,22 @@ void windows_path_join(SEXP x, int x_length, int commonLength, SEXP value)
 
 #if debug
         if (check_width != pwidth) Rf_error("allocated string of %d bytes, allocated %d bytes instead\n", pwidth, check_width);
-        if (strlen(cbuf) != pwidth) Rf_error("allocated string of %d bytes, got a string of %d bytes instead\n", pwidth, strlen(cbuf));
+        if ((int) strlen(cbuf) != pwidth) Rf_error("allocated string of %d bytes, got a string of %d bytes instead\n", pwidth, (int) strlen(cbuf));
         Rprintf("\n");
 #endif
 
 
+        cetype_t enc = CE_NATIVE;
+        if (use_Bytes) enc = CE_BYTES;
+        else if (use_UTF8) enc = CE_UTF8;
+        else if (anyKnown && allKnown) {
+            if (is_latin1locale) enc = CE_LATIN1;
+            if (is_utf8locale) enc = CE_UTF8;
+        }
+
+
         /* use cbuf because we want the whole string */
-        SET_STRING_ELT(value, j, Rf_mkCharCE(cbuf, CE_UTF8));
+        SET_STRING_ELT(value, j, Rf_mkCharCE(cbuf, enc));
     }
 }
 
@@ -445,50 +571,47 @@ void unix_path_join(SEXP x, int x_length, int commonLength, SEXP value)
     int pwidth;
 
 
-    const char *ptr;
-    int *need_trailing_slash;
-    need_trailing_slash = (int *) R_alloc(x_length, sizeof(int));
+    const char *str;
+    info_t *info;
+    info = (info_t *) R_alloc(x_length, sizeof(info_t));
 
 
     int len, nchar;
 
 
+    Rboolean is_latin1locale = my_latin1locale,
+             is_utf8locale   = my_utf8locale  ;
+
+
+    Rboolean allKnown, anyKnown, use_UTF8, use_Bytes;
+
+
+    SEXP cs;
+
+
     for (j = 0; j < commonLength; j++) {
 
 
-        pwidth = 0;
+        memset(info, 0, x_length * sizeof(info_t));
 
 
-        ptr = "";
-        memset(need_trailing_slash, 0, x_length * sizeof(int));
+        anyKnown = FALSE; allKnown = TRUE; use_UTF8 = FALSE; use_Bytes = FALSE;
 
 
-        /* we're going to work backwards here
-         * since we want the LAST absolute path
+        /* find the desired encoding
+         * and the index of the last absolute path
+         *
+         * we're going to work backwards here
          */
         for (i = x_length - 1; i >= 0; i--) {
             len = LENGTH(VECTOR_ELT(x, i));
-            ptr = Rf_translateCharUTF8(STRING_ELT(VECTOR_ELT(x, i), j % len));
-            nchar = (int) strlen(ptr);
-            if (!nchar)
-                continue;
-
-
-            /* if there are characters already in the buffer
-             * and this path does not end with /
-             * then we will need to add one manually,
-             * record this index as needing a trailing slash
-             */
-            if (pwidth && ptr[nchar - 1] != '/') {
-                need_trailing_slash[i] = 1;
-                pwidth++;
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            if (!use_Bytes) {
+                if (IS_BYTES(cs)) use_Bytes = TRUE;
+                else if (!use_UTF8 && IS_UTF8(cs)) use_UTF8 = TRUE;
             }
 
 
-            /* add this string to the width of the buffer */
-            pwidth += nchar;
-
-
             if (i == 0) break;
 
 
@@ -497,15 +620,54 @@ void unix_path_join(SEXP x, int x_length, int commonLength, SEXP value)
              * if it is equal to ~
              * if it starts with ~/
              */
-            if (*ptr == '/') break;  /* path starts with / */
-            if (*ptr == '~') {
-                if (nchar == 1) break;  /* path equals ~ */
-                if (*(ptr + 1) == '/') break;  /* path starts with ~/ */
-                if (*R_ExpandFileName(ptr) == '/') break;  /* path expands to an absolute path, e.g. ~iris/foo might expand to /home/iris/foo */
+            str = R_CHAR(cs);
+            if (*str == '/') break;  /* path starts with / */
+            if (*str == '~') {
+                if (*(str + 1) == '\0') break;  /* path equals ~       */
+                if (*(str + 1) == '/' ) break;  /* path starts with ~/ */
             }
         }
 
 
+        int i_min = (i > 0) ? i : 0;
+
+
+        pwidth = 0;
+
+
+        /* now that we have the desired encoding and the starting index,
+         * find the required width
+         */
+        for (i = x_length - 1; i >= i_min; i--) {
+            len = LENGTH(VECTOR_ELT(x, i));
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            if (use_Bytes)
+                str = R_CHAR(cs);
+            else if (use_UTF8)
+                str = Rf_translateCharUTF8(cs);
+            else
+                str = Rf_translateChar(cs);
+            nchar = (int) strlen(str);
+            if (!nchar)
+                continue;
+
+
+            /* if there are characters already in the buffer
+             * and this path does not end with /
+             * then we will need to add one manually,
+             * record this index as needing a trailing slash
+             */
+            if (pwidth && str[nchar - 1] != '/') {
+                SET_NEED_TRAILING_SLASH(info[i]);
+                pwidth++;
+            }
+
+
+            /* add this string to the width of the buffer */
+            pwidth += nchar;
+        }
+
+
         /* if all the strings were empty, return an empty string */
         if (pwidth <= 0) {
             // SET_STRING_ELT(value, j, R_BlankString);
@@ -531,11 +693,21 @@ void unix_path_join(SEXP x, int x_length, int commonLength, SEXP value)
 
 
         /* take where we exited and start from there */
-        if (i < 0) i = 0;
-        for (; i < x_length; i++) {
+        for (i = i_min; i < x_length; i++) {
             len = LENGTH(VECTOR_ELT(x, i));
-            ptr = Rf_translateCharUTF8(STRING_ELT(VECTOR_ELT(x, i), j % len));
-            nchar = (int) strlen(ptr);
+            cs = STRING_ELT(VECTOR_ELT(x, i), j % len);
+            if (use_Bytes)
+                str = R_CHAR(cs);
+            else if (use_UTF8)
+                str = Rf_translateCharUTF8(cs);
+            else {
+                str = Rf_translateChar(cs);
+                // allKnown = allKnown && (IS_ASCII(cs) || ENC_KNOWN(cs));
+                // anyKnown = anyKnown || ENC_KNOWN(cs);
+                allKnown = allKnown && (IS_ASCII(cs) || IS_LATIN1(cs));
+                anyKnown = anyKnown || IS_LATIN1(cs);
+            }
+            nchar = (int) strlen(str);
 
 
             /* an empty string does not need to be copied to the buffer
@@ -545,20 +717,20 @@ void unix_path_join(SEXP x, int x_length, int commonLength, SEXP value)
                 continue;
 
 
-            strcpy(buf, ptr);
+            strcpy(buf, str);
             buf += nchar;
 #if debug
-                Rprintf("copied to buffer: %s", ptr); Rprintf("\n");
+                Rprintf("copied to buffer: %s", str); Rprintf("\n");
                 Rprintf("shifted buffer by %d bytes\n", nchar);
-                check_width += strlen(ptr);
+                check_width += (int) strlen(str);
 #endif
-            if (need_trailing_slash[i]) {
+            if (NEED_TRAILING_SLASH(info[i])) {
                 strcpy(buf, "/");
                 buf++;
 #if debug
                     Rprintf("copied to buffer: %s", "/"); Rprintf("\n");
-                    Rprintf("shifted buffer by %d bytes\n", strlen("/"));
-                    check_width += strlen("/");
+                    Rprintf("shifted buffer by %d bytes\n", (int) strlen("/"));
+                    check_width += (int) strlen("/");
 #endif
             }
         }
@@ -566,13 +738,22 @@ void unix_path_join(SEXP x, int x_length, int commonLength, SEXP value)
 
 #if debug
         if (check_width != pwidth) Rf_error("allocated string of %d bytes, allocated %d bytes instead\n", pwidth, check_width);
-        if (strlen(cbuf) != pwidth) Rf_error("allocated string of %d bytes, got a string of %d bytes instead\n", pwidth, strlen(cbuf));
+        if ((int) strlen(cbuf) != pwidth) Rf_error("allocated string of %d bytes, got a string of %d bytes instead\n", pwidth, (int) strlen(cbuf));
         Rprintf("\n");
 #endif
 
 
+        cetype_t enc = CE_NATIVE;
+        if (use_Bytes) enc = CE_BYTES;
+        else if (use_UTF8) enc = CE_UTF8;
+        else if (anyKnown && allKnown) {
+            if (is_latin1locale) enc = CE_LATIN1;
+            if (is_utf8locale) enc = CE_UTF8;
+        }
+
+
         /* use cbuf because we want the whole string */
-        SET_STRING_ELT(value, j, Rf_mkCharCE(cbuf, CE_UTF8));
+        SET_STRING_ELT(value, j, Rf_mkCharCE(cbuf, enc));
     }
 }
 
@@ -604,7 +785,7 @@ SEXP path_join(SEXP call, int windows, const char *name, SEXP args, SEXP rho)
     SEXP x = Rf_allocVector(VECSXP, dots_length);
     Rf_protect(x); nprotect++;
     int x_length = dots_length;
-    int i, j;
+    int i;
     SEXP d, xi;
 
 
@@ -673,16 +854,6 @@ SEXP path_join(SEXP call, int windows, const char *name, SEXP args, SEXP rho)
     }
 
 
-    for (i = 0; i < x_length; i++) {
-        int len = LENGTH(VECTOR_ELT(x, i));
-        for (j = 0; j < len; j++) {
-            SEXP cs = STRING_ELT(VECTOR_ELT(x, i), j);
-            if (Rf_getCharCE(cs) == CE_BYTES)
-                Rf_error("strings with \"bytes\" encoding are not allowed");
-        }
-    }
-
-
     SEXP value = Rf_allocVector(STRSXP, commonLength);
     Rf_protect(value); nprotect++;
 
